<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAShippingSitePlugin.php');
require_once($libpath . 'CrBcShippingSitePlugin.php');

class CrBc_Plugins_Shipping_Extendedshipping_Site extends CrBcAShippingSitePlugin implements CrBcShippingSitePlugin
{
    private $cart = null;
	//new
	private $options = null;
    private $freeShippingProducts = null;
	private $_order_info = null;
	private $_cart_items = null;
	private $country_id = null;
	private $total_net = null;
    
    public function  __construct() {
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        
        // always call the parent constructor and always call it _first_
        parent::__construct();
        // define the default table for built-in list/details view
        $this->table = '#__breezingcommerce_plugin_extendedshipping';
    }
    
    function init($subject = NULL){
        if( !($subject instanceof CrBcCart) ){ throw new Exception('init subject must be of type CrBcCart'); }
        $this->cart = $subject;
		//new
		$total_net = 0;
        
		if( $this->cart->currency_conversion_rate > 0 ){
		
			try{

				$total_net = CrBcCart::getOrderInfoFromCart()['info']->price_net / $this->cart->currency_conversion_rate;

			} catch(Exception $e){

				return '';
			}
		
		}else{
			
			return '';
		}
        
        $db = JFactory::getDBO();
        $db->setQuery("Select * From #__breezingcommerce_plugin_extendedshipping Where published = 1 Order By `rate`");
        $options = $db->loadAssocList();
        
		$_session_cart = JFactory::getSession()->get('crbc_cart', array());
		
		$customer = null;
		
		if(isset($_session_cart['customer_id']) && intval($_session_cart['customer_id']) > 0){
		
			$db->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($_session_cart['customer_id']) . " Limit 1");
			$customer = $db->loadObject();

		}
		
		$country_id = 0;
        if($customer) $country_id = $customer->country_id;
		
		$_cart = new CrBcCart( $_session_cart );
			
		$_cart_items = $_cart->getItems(true);        
		$data = CrBcCart::getData($_session_cart['order_id'], $_cart_items, -1, -1);
		$_order_info = CrBcCart::getOrder(
					$_session_cart['order_id'], 
					$_cart, 
					$_session_cart, 
					$_cart_items, 
					$_session_cart['customer_id'],
					$data,
					array(),
					array()
			);
		
		if( 
			$customer !== null 
			&&
			isset($_session_cart['checkout'])
			&& 
			( 
				isset($_session_cart['checkout']['.crbc-checkout-billing-information_done']) 
				|| 
				isset($_session_cart['checkout']['.crbc-checkout-shipping-information_done']) 
			) 
		){
			
			// now we need to know if shipping address shall be used or the billing address
			$shipping = '';
			if($customer->use_shipping_address){
					$shipping = 'shipping_';
			} else {
					$shipping = '';
			}

			// depending on that, we pull the right country id from the checkout address selection
			$country_id = $customer->{$shipping.'country_id'};
			
		} else {
			
		   // getting defaults based on location services or default group
			$group_data = null;

			if( $customer !== null ){
			
				$db->setQuery("
					Select
						groups.*
					From 
						#__breezingcommerce_customers As customers, 
						#__breezingcommerce_customergroups As groups 
					Where 
						customers.id = " . $db->quote($customer->id) . "
					And 
						customers.customergroup_id = groups.id 
					And 
						customers.published = 1 
					And 
						groups.published = 1");

				$group_data = $db->loadObject();

			} else if(intval(JFactory::getUser()->get('id',0)) != 0){

				$db->setQuery("Select Distinct groups.* From #__breezingcommerce_customergroups groups, #__breezingcommerce_customers customers Where groups.published = 1 And customers.published = 1 And customers.customergroup_id = groups.id And customers.userid = " . $db->quote(intval(JFactory::getUser()->get('id',0))) . " Limit 1");
				$group_data = $db->loadObject();

			} else {
				// let's see if there is one determinable by the user's location
				$group_data = CrBcHelpers::getBestCustomerGroupByLocation(true);
			}

			// still nothing, try the default group
			if( !$group_data ){
				
				$db->setQuery("Select id, default_country_id, default_region_id From #__breezingcommerce_customergroups Where `default` = 1");
				$group_data = $db->loadObject();
			}

			$default_customer_group_id = 0;
			$default_country_id = 0;
			$default_region_id = 0;

			if( $group_data ){
				
				$default_customer_group_id = $group_data->id;
				$default_country_id = $group_data->default_country_id;
				$default_region_id = $group_data->default_region_id;
			}
			
			if($country_id == 0) $country_id = $default_country_id;
		}
		
		list($freeShippingProducts, $rate) = $this->getCompanionRules($country_id);
		if($rate instanceof stdClass){
			$options = array();
			$options[0]['identity'] = $rate->identity;
			$options[0]['producttaxclass_id'] = $rate->producttaxclass_id;
			$options[0]['key'] = $rate->title;
			$options[0]['countries'] = $rate->countries;
			$options[0]['from_weight'] = '0';
			$options[0]['to_weight'] = '0';
			$options[0]['weight_type'] = '0';
			$options[0]['rate'] = $rate->rate;
			$options[0]['from_total_net'] = '0.0000';
			$options[0]['to_total_net'] = '0.0000';
		}
		
		$this->options = $options;
        $this->freeShippingProducts = $freeShippingProducts;
		$this->_order_info = $_order_info;
		$this->_cart_items = $_cart_items;
		$this->country_id = $country_id;
		$this->total_net = $total_net;
    }
    
    function getOptionElementName(){
        return 'crbc_shipping_extendedshipping';
    }
    
    public function getPluginDisplayName(){
        return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_EXTENDEDSHIPPING');
    }
    
    function validateSelectedOption($option, $order_id){
        if(intval($option) <= 0){
            return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_EXTENDEDSHIPPING_SHIPPING_PLEASE_SELECT_AN_OPTION');
        }
        return '';
    }
    
    function onOptionUpdate($option, $order_id){
        // nothing required here
    }
    
    function determineBestShippingOptions(){
        // nothing, flatrate doesn't care
    }
	
	function getCompanionRules($country_id){
		$db = JFactory::getDBO();
        $db->setQuery("Select id From #__breezingcommerce_plugins Where type = 'productdetails' And name = 'extendedshipping' And published = 1 Limit 1;");
        $companionPlugin = $db->loadResult();
		if(!$companionPlugin) return array(1,1);
		$productIds = array();
		foreach($this->cart->cart['products'] as $product){
			if((!$product['is_combination'] /*|| !isset($product['properties'][0])*/) && !isset($productIds[$product['id'].'-0'])){
				$productIds[$product['id'].'-0']['amount'] = $product['amount'];
				$db->setQuery("Select weight, weight_type, title From #__breezingcommerce_products Where id = " . $product['id'] . ";");
				$_product = $db->loadRow();
				$productIds[$product['id'].'-0']['weight'] = $_product[0];
				$productIds[$product['id'].'-0']['weight_type'] = $_product[1];
				$productIds[$product['id'].'-0']['title'] = $_product[2];
			} elseif(!$product['is_combination']) {
				$productIds[$product['id'].'-0']['amount'] += $product['amount'];
			}
			
			$property = isset($product['properties'][0]) ? $product['properties'][0]['id'] : 0;
			if(($product['is_combination'] /*&& isset($product['properties'][0])*/) && !isset($productIds[$product['id'].'-'.$property])){
				$productIds[$product['id'].'-'.$property]['amount'] = $product['amount'];
				$db->setQuery("Select weight, weight_type, title From #__breezingcommerce_products Where id = " . $product['id'] . ";");
				$_product = $db->loadRow();
				$productIds[$product['id'].'-'.$property]['weight'] = $_product[0];
				$productIds[$product['id'].'-'.$property]['weight_type'] = $_product[1];
				$productIds[$product['id'].'-'.$property]['title'] = $_product[2];
			} elseif($product['is_combination']) {
				$productIds[$product['id'].'-'.$property]['amount'] += $product['amount'];
			}
		}
		
		$i = 0;
		$freeShippingProducts = array();
		$rate = false;
		foreach($productIds as $key => $value){
			if($i > 1) return true;
			list($product_id, $combination_id) = explode("-", $key);
			$db->setQuery("Select * From #__breezingcommerce_plugin_extendedshippingcompanion Where product_id = " . intval($product_id) . " And combination_id = " . intval($combination_id) . " And published = 1 Order By `identity` Desc Limit 1;");
			$rule = $db->loadObject();
			
			if($rule && !$rule->free_shipping && in_array($country_id, explode(",", $rule->countries))){
				$rate = new stdClass();
				$rate->rate = $rule->rate;
				$rate->title = $productIds[$key]['title'];
				$rate->countries = $rule->countries;
				$rate->identity = $rule->identity;
				$rate->producttaxclass_id = $rule->producttaxclass_id;
				$i++;
			} elseif($rule && $rule->free_shipping && in_array($country_id, explode(",", $rule->countries))) {
				$freeShippingProducts[] = $key;
			} elseif(!$rule) {
				$i++;			
			}
		}

		if($i <> 1){
			return array($freeShippingProducts, false);
		} else {
		
			return array($freeShippingProducts, $rate);		
		}		
    }
    
    function onDisplayOptions($selected_key){
		$options = $this->options;
        $freeShippingProducts = $this->freeShippingProducts;
		$_order_info = $this->_order_info;
		$_cart_items = $this->_cart_items;
		$country_id = $this->country_id;
		$total_net = $this->total_net; 
		
		$new_options = array();
		$config = CrBcHelpers::getBcConfig();
		$weightType = $config->get('weight_type');
		if(is_array($options) && count($options)){
            foreach($options As $option){
				$subtractedWeight = 0;
				if($freeShippingProducts != 1 && !empty($freeShippingProducts)){
					$x = 0;					
					foreach($_order_info->cart_instance->cart['products'] as $item){
						if(!in_array($item['id'].'-'.($item['is_combination'] && isset($item['properties'][0]) ? $item['properties'][0]['id'] : 0), $freeShippingProducts)) continue; 
						$cartItem = $_cart_items[$x];
						$cartItem->weight = $cartItem->weight * $cartItem->amount;
						
						if($weightType == 0){
							if( $cartItem->weight_type == '0' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= 1000 * $cartItem->weight;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= 1000000 * $cartItem->weight;
								}
							}
							if( $cartItem->weight_type == '1' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight / 1000;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= $cartItem->weight;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= 1000 * $cartItem->weight;
								}			
							}
							if( $cartItem->weight_type == '2' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight / 1000000;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= $cartItem->weight / 1000;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= $cartItem->weight;
								}			
							}
						} elseif($weightType == 1){
							if( $cartItem->weight_type == '0' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= 16 * $cartItem->weight;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= 16 * 437.5 * $cartItem->weight;
								}
							}
							if( $cartItem->weight_type == '1' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight / 16;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= $cartItem->weight;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= 437.5 * $cartItem->weight;
								}			
							}
							if( $cartItem->weight_type == '2' ){
								if($option['weight_type'] == 0){
									$subtractedWeight -= $cartItem->weight / 16 / 437.5;
								} elseif($option['weight_type'] == 1){
									$subtractedWeight -= $cartItem->weight / 437.5;
								} elseif($option['weight_type'] == 2){
									$subtractedWeight -= $cartItem->weight;
								}			
							}
						}
						$x++;
					}
				}		
			
				$weight = 0;
				if($weightType == 0){
					if( isset($_order_info->weight[0]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[0];
						} elseif($option['weight_type'] == 1){
							$weight += 1000 * $_order_info->weight[0];
						} elseif($option['weight_type'] == 2){
							$weight += 1000000 * $_order_info->weight[0];
						}
					}
					if( isset($_order_info->weight[1]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[1] / 1000;
						} elseif($option['weight_type'] == 1){
							$weight += $_order_info->weight[1];
						} elseif($option['weight_type'] == 2){
							$weight += 1000 * $_order_info->weight[1];
						}			
					}
					if( isset($_order_info->weight[2]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[2] / 1000000;
						} elseif($option['weight_type'] == 1){
							$weight += $_order_info->weight[2] / 1000;
						} elseif($option['weight_type'] == 2){
							$weight += $_order_info->weight[2];
						}			
					}
				} elseif($weightType == 1){
					if( isset($_order_info->weight[0]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[0];
						} elseif($option['weight_type'] == 1){
							$weight += 16 * $_order_info->weight[0];
						} elseif($option['weight_type'] == 2){
							$weight += 16 * 437.5 * $_order_info->weight[0];
						}
					}
					if( isset($_order_info->weight[1]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[1] / 16;
						} elseif($option['weight_type'] == 1){
							$weight += $_order_info->weight[1];
						} elseif($option['weight_type'] == 2){
							$weight += 437.5 * $_order_info->weight[1];
						}			
					}
					if( isset($_order_info->weight[2]) ){
						if($option['weight_type'] == 0){
							$weight += $_order_info->weight[2] / 16 / 437.5;
						} elseif($option['weight_type'] == 1){
							$weight += $_order_info->weight[2] / 437.5;
						} elseif($option['weight_type'] == 2){
							$weight += $_order_info->weight[2];
						}			
					}
				}

				$weight += $subtractedWeight;
                if((!isset($option['countries']) || in_array($country_id, explode(",", $option['countries']))) && ($option['from_weight'] <= $weight) && ($weight <= $option['to_weight'] || ( $weight > $option['to_weight'] && $option['to_weight'] == 0 ) ) && ($option['from_total_net'] <= $total_net) && ($total_net <= $option['to_total_net'] || ( $total_net > $option['to_total_net'] && $option['to_total_net'] == 0 ) ) ){
                    
                    $new_options[] = $option;
                }
            }
		}	
            if( count( $new_options ) > 1 ){
            
                $out = '<select name="'.$this->getOptionElementName().'">'."\n";
                foreach($new_options As $option){

                    CrBcCart::$exclude_from_no_tax_list[] = $option['producttaxclass_id'];
                    $result = CrBcCart::getPrice($option['rate'], $this->cart->currency_conversion_rate, $option['producttaxclass_id'], 1, $this->cart->cart['customer_id']);
                    
                    $translated_object = CrBcHelpers::loadTranslation($option['identity'], 'plugin_shipping_extendedshipping');
                    if($translated_object !== null){
                        $option['key'] = $translated_object->title;
                    }

                    $out .= '<option value="'.$option['identity'].'"'.($option['identity'] == $selected_key ? ' selected="selected"' : '').'>'.$option['key'].' ('.$this->cart->formatPrice($result['gross']).')</option>'."\n";
                }
                return $out."\n".'</select>'."\n";
                
            }
            else
            if( count( $new_options ) == 1 ){
                
                CrBcCart::$exclude_from_no_tax_list[] = $new_options[0]['producttaxclass_id'];
                $result = CrBcCart::getPrice($new_options[0]['rate'], $this->cart->currency_conversion_rate, $new_options[0]['producttaxclass_id'], 1, $this->cart->cart['customer_id']);
                
                $translated_object = CrBcHelpers::loadTranslation($new_options[0]['identity'], 'plugin_shipping_extendedshipping');
                if($translated_object !== null){
                    $option['key'] = $translated_object->title;
                }
                
                $out = $new_options[0]['key'].' ('.$this->cart->formatPrice($result['gross']).')';
                $out .= '<div><i class="crbc-fa crbc-fa-truck"></i> '.JText::_('COM_BREEZINGCOMMERCE_PLUGIN_EXTENDEDSHIPPING_BASED_ON').'</div>';
                return $out . '<input type="hidden" value="'.$new_options[0]['identity'].'" name="'.$this->getOptionElementName().'" />';
            }

        return '';
    }
    
    function getProductTaxclassId($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select producttaxclass_id From #__breezingcommerce_plugin_extendedshipping Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       } else {
		    $db->setQuery("Select id From #__breezingcommerce_plugins Where type = 'productdetails' And name = 'extendedshipping' And published = 1 Limit 1;");
            $companionPlugin = $db->loadResult();
			if($companionPlugin){
				$db->setQuery("Select producttaxclass_id From #__breezingcommerce_plugin_extendedshippingcompanion Where published = 1 And identity = ".intval($option));
				$result = $db->loadResult();
				if($result) return $result;
			}
	   }
       return 0;
    }
    
    function getPrice($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select rate From #__breezingcommerce_plugin_extendedshipping Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result * $this->cart->currency_conversion_rate;
       } else {
		    $db->setQuery("Select id From #__breezingcommerce_plugins Where type = 'productdetails' And name = 'extendedshipping' And published = 1 Limit 1;");
            $companionPlugin = $db->loadResult();
			if($companionPlugin){
				$db->setQuery("Select rate From #__breezingcommerce_plugin_extendedshippingcompanion Where published = 1 And identity = ".intval($option));
				$result = $db->loadResult();
				if($result) return $result * $this->cart->currency_conversion_rate;
			}
	   }
       return 0;
    }
    
    function getName($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select `key` From #__breezingcommerce_plugin_extendedshipping Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       } else {
		    $db->setQuery("Select id From #__breezingcommerce_plugins Where type = 'productdetails' And name = 'extendedshipping' And published = 1 Limit 1;");
            $companionPlugin = $db->loadResult();
			if($companionPlugin){
				$db->setQuery("Select title, rate From #__breezingcommerce_plugin_extendedshippingcompanion Where published = 1 And identity = ".intval($option));
				$result = $db->loadObject();
				if($result) return $result->title.' ('.$this->cart->formatPrice($result->rate).')';
			}
	   }
       return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_EXTENDEDSHIPPING_UNKNOWN');
    }
}